
package rcpsp_choco_base;

import org.chocosolver.solver.*;
import org.chocosolver.solver.variables.IntVar;
import org.chocosolver.util.ESat;

public class RCPSP_Choco_base {
 
    public static void main(String[] args) {
     
     Model mon_modele = new Model("RCPSP");          

        int n = 7;
        int Dmax = 22;
        int Qdispo = 4;
        int les_durees[] = new int [100] ;
        int les_conso[] = new int [100] ;
        int pred[][] = new int [100][100] ;              
               
        
        // jeux de donnees 1 :
        les_durees[0]=0;
        les_durees[1]=10;
        les_durees[2]=4;
        les_durees[3]=5;
        les_durees[4]=12;
        les_durees[5]=10;
        les_durees[6]=0;
        
            
        les_conso[0]=0;
        les_conso[1]=2;
        les_conso[2]=1;
        les_conso[3]=3;
        les_conso[4]=2;
        les_conso[5]=1;
        les_conso[6]=Qdispo;
  
        
        for (int i=0; i<100; i++){
            for (int j=0; j<100; j++){
              pred[i][j] = 0;
           }               
        }
        pred[1][0]=1;
        pred[2][0]=1;
        pred[3][0]=1;
        pred[4][0]=1;
        pred[5][0]=1;
        pred[6][0]=1;
        
        pred[6][0]=1;
        pred[6][1]=1;
        pred[6][2]=1;
        pred[6][3]=1;
        pred[6][4]=1;
        pred[6][5]=1;
        
        pred[4][3]=1;


        
        // contrainte 1
        IntVar[] St = new IntVar[n];
        for (int i=0; i<n; i++){
            St[i] = mon_modele.intVar("St_"+i, 0, Dmax);
        }

        // contrainte 2
        IntVar[] Ft = new IntVar[n];
        for (int i=0; i<n; i++){
            Ft[i] = mon_modele.intVar("Ft_"+i,0, Dmax); 
        }

        
        // contrainte 3  
        IntVar[][] qt = new IntVar[n][n];
        for (int i=0; i<n; i++){
            for (int j=0; j<n; j++){
                qt[i][j] = mon_modele.intVar("qt_"+i+"_"+j, 0, Qdispo);
            }
        } 
 
        // contrainte 4
        IntVar OBJ = mon_modele.intVar("Cmax", 0, Dmax);      
                
        // contrainte 5
        for (int i=0; i<n; i++){
            mon_modele.arithm(Ft[i],"=",St[i],"+",les_durees[i]).post();
        }
        
        // contrainte 6
        mon_modele.sum(qt[0],"=",Qdispo).post();
        
        // contrainte 7
        int[] coeffs = new int[n];
        for (int j=0; j<n ; j++){
            coeffs[j] = 1;  
        }
        for (int i=1; i<n-1; i++){
            IntVar[] colonne = new IntVar[n];            
            for (int j=0; j<n ; j++){
               colonne[j] = qt[j][i];  
             }
             mon_modele.scalar(colonne, coeffs,"=",les_conso[i]).post();
        }

        // contrainte 8
        IntVar[] colonne = new IntVar[n];            
        for (int j=0; j<n ; j++){
            colonne[j] = qt[j][n-1];  
        }
        mon_modele.scalar(colonne, coeffs,"=",Qdispo).post();     


        // contrainte 9
        for (int i=1; i<n-1; i++){
            mon_modele.sum(qt[i],"=",les_conso[i]).post();
        }

        // contrainte 10
        for (int i=0; i<n; i++){
            for (int j=0; j<n; j++) {            
               mon_modele.ifThen(
                    mon_modele.arithm(qt[i][j],">",0),
                    mon_modele.arithm(St[j],">=",Ft[i]));              
            }
        } 
        
        // contrainte 11
        for (int i=0; i<n; i++){
            for (int j=0; j<n; j++)  {            
              if (pred[i][j]==1) {
                mon_modele.arithm(St[i],">=",Ft[j]).post();
              }
            }
        }
      
        // contrainte 12
        OBJ=Ft[n-1];
                
        // fonction objectif
        
        mon_modele.setObjective(Model.MINIMIZE, OBJ); 

       
        Solver mon_solveur = mon_modele.getSolver();
        Solution s = new Solution(mon_modele);
     

        while (mon_solveur.solve()) {
            s.record();
            int cout = s.getIntVal(St[n-1]);
            System.out.println("Cmax = " + cout);
        }  
   
        System.out.println("*****************");
        if (mon_solveur.isFeasible() == ESat.TRUE){
            int cout = s.getIntVal(St[n-1]);
            System.out.println("Cmax = " + cout);
            System.out.println("Solution = ");

            for (int i=0; i<n; i++){
                System.out.print( ""+i+" : "+s.getIntVal(St[i]) + " - ");
                System.out.println(s.getIntVal(Ft[i]) + " ");
            }

            System.out.println( "Flot  :");
            for (int i=0; i<n-1; i++){
                for (int j=1; j<n; j++){
                    if (s.getIntVal(qt[i][j])>0){
                        System.out.println(""+i+" - "+j+" : "+ s.getIntVal(qt[i][j]));
                    }
                }
            } 
        }
        else{
            System.out.println("pas de sol.faisable");}
                 
    }   
    
}
